@@ -5,15 +5,14 @@ use warnings;
 
 # Added by eilara as hack around caller() core dump
 # NOTE: Now we've switched to Sub::Uplevel can this be removed? --ADAMK
-use Carp::Heavy                   (); 
-use Carp                          ();
-use Sub::Uplevel                  ();
-use Aspect::Hook                  ();
-use Aspect::Advice::After         ();
-use Aspect::Point::AfterReturning ();
+use Carp::Heavy           (); 
+use Carp                  ();
+use Sub::Uplevel          ();
+use Aspect::Advice        ();
+use Aspect::AdviceContext ();
 
-our $VERSION = '0.97_05';
-our @ISA     = 'Aspect::Advice::After';
+our $VERSION = '0.34';
+our @ISA     = 'Aspect::Advice';
 
 # NOTE: To simplify debugging of the generated code, all injected string
 # fragments will be defined in $UPPERCASE, and all lexical variables to be
@@ -28,9 +27,8 @@ sub _install {
 	# runtime checks instead of the original.
 	# Because $MATCH_RUN is used in boolean conditionals, if there
 	# is nothing to do the compiler will optimise away the code entirely.
-	my $curried   = $pointcut->curry_runtime;
-	my $compiled  = $curried ? $curried->compiled_runtime : undef;
-	my $MATCH_RUN = $compiled ? '$compiled->()' : 1;
+	my $curried   = $pointcut->curry_run;
+	my $MATCH_RUN = $curried ? '$curried->match_run($name, $runtime)' : 1;
 
 	# When an aspect falls out of scope, we don't attempt to remove
 	# the generated hook code, because it might (for reasons potentially
@@ -63,12 +61,11 @@ sub _install {
 		# Generate the new function
 		no warnings 'redefine';
 		eval <<"END_PERL"; die $@ if $@;
-		package Aspect::Hook;
-
 		*$NAME = sub $PROTOTYPE {
 			# Is this a lexically scoped hook that has finished
 			goto &\$original if $MATCH_DISABLED;
 
+			my \$runtime   = {};
 			my \$wantarray = wantarray;
 			if ( \$wantarray ) {
 				my \$return = [
@@ -76,68 +73,78 @@ sub _install {
 						1, \$original, \@_,
 					)
 				];
-
-				local \$_ = bless {
-					sub_name     => \$name,
-					wantarray    => \$wantarray,
-					args         => \\\@_,
-					return_value => \$return,
-					pointcut     => \$pointcut,
-					original     => \$original,
-				}, 'Aspect::Point::AfterReturning';
-
 				return \@\$return unless $MATCH_RUN;
 
+				# Create the context
+				my \$context = Aspect::AdviceContext->new(
+					type         => 'after_returning',
+					pointcut     => \$pointcut,
+					sub_name     => \$name,
+					wantarray    => \$wantarray,
+					params       => \\\@_,
+					return_value => \$return,
+					original     => \$original,
+					\%\$runtime,
+				);
+
 				# Execute the advice code
-				&\$code(\$_);
+				() = &\$code(\$context);
 
 				# Get the (potentially) modified return value
-				return \@{\$_->{return_value}};
+				\$return = \$context->return_value;
+				if ( ref \$return eq 'ARRAY' ) {
+					return \@\$return;
+				} else {
+					return ( \$return );
+				}
 			}
 
 			if ( defined \$wantarray ) {
 				my \$return = Sub::Uplevel::uplevel(
 					1, \$original, \@_,
 				);
-
-				local \$_ = bless {
-					sub_name     => \$name,
-					wantarray    => \$wantarray,
-					args         => \\\@_,
-					return_value => \$return,
-					pointcut     => \$pointcut,
-					original     => \$original,
-				}, 'Aspect::Point::AfterReturning';
-
 				return \$return unless $MATCH_RUN;
 
+				# Create the context
+				my \$context = Aspect::AdviceContext->new(
+					type         => 'after_returning',
+					pointcut     => \$pointcut,
+					sub_name     => \$name,
+					wantarray    => \$wantarray,
+					params       => \\\@_,
+					return_value => \$return,
+					original     => \$original,
+					\%\$runtime,
+				);
+
 				# Execute the advice code
-				&\$code(\$_);
-				return \$_->{return_value};
+				my \$dummy = &\$code(\$context);
+				return \$context->return_value;
 
 			} else {
 				Sub::Uplevel::uplevel(
 					1, \$original, \@_,
 				);
-
-				local \$_ = bless {
-					sub_name     => \$name,
-					wantarray    => \$wantarray,
-					args         => \\\@_,
-					return_value => undef,
-					pointcut     => \$pointcut,
-					original     => \$original,
-				}, 'Aspect::Point::AfterReturning';
-
 				return unless $MATCH_RUN;
 
+				# Create the context
+				my \$context = Aspect::AdviceContext->new(
+					type         => 'after_returning',
+					pointcut     => \$pointcut,
+					sub_name     => \$name,
+					wantarray    => \$wantarray,
+					params       => \\\@_,
+					return_value => undef,
+					original     => \$original,
+					\%\$runtime,
+				);
+
 				# Execute the advice code
-				&\$code(\$_);
+				&\$code(\$context);
 				return;
 			}
 		};
 END_PERL
-		$self->{installed}++;
 	}
 
 	# If this will run lexical we don't need a descoping hook
@@ -150,65 +157,4 @@ END_PERL
 	return sub { $out_of_scope = 1 };
 }
 
-# Check for pointcut usage not supported by the advice type
-sub _validate {
-	my $self     = shift;
-	my $pointcut = $self->pointcut;
-
-	# Pointcuts using "throwing" are irrelevant in before advice
-	if ( $pointcut->match_contains('Aspect::Pointcut::Throwing') ) {
-		return 'The pointcut throwing is illegal when used by after_returning advice';
-	}
-
-	$self->SUPER::_validate(@_);
-}
-
 1;
-
-__END__
-
-=pod
-
-=head1 NAME
-
-Aspect::Advice::AfterReturning - Execute code when a function returns
-without throwing an exception.
-
-=head1 SYNOPSIS
-
-  use Aspect;
-  
-  after_returning {
-      # Trace all normally returning calls to your module
-      print STDERR "Called my function " . $_->sub_name . "\n";
-  
-      # Throw an exception if foo() returns 'bar'
-      if ( $_->short_name eq 'foo' and $_->return_value eq 'bar' ) {
-          $_->exception("Missing or invalid arguments to foo()");
-      }
-  
-  } call qr/^ MyModule::\w+ $/
-
-=head1 DESCRIPTION
-
-The C<after_returning> advice type is used to execute code after a function
-has returned, but C<WITHOUT> trapping exceptions (they will propogate
-upwards normally).
-
-As well as creating side effects that run after the main code, the
-C<after_returning> advice type is particularly useful for selectively
-altering the return value of a function or API-shifting functions which
-signal errors by returning C<undef> so they throw exceptions instead.
-
-=head1 AUTHORS
-
-Adam Kennedy E<lt>adamk@cpan.orgE<gt>
-
-=head1 COPYRIGHT AND LICENSE
-
-Copyright 2010 Adam Kennedy.
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
-
-=cut
